package com.cultural.heritage.controller.order;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.model.dto.order.BookingOrderQueryRequest;
import com.cultural.heritage.model.entity.PendingServiceGood;
import com.cultural.heritage.model.entity.PendingServiceOrder;
import com.cultural.heritage.model.vo.order.PendingServiceGoodVO;
import com.cultural.heritage.model.vo.order.PendingServiceOrderVO;
import com.cultural.heritage.service.order.PendingServiceGoodService;
import com.cultural.heritage.service.order.PendingServiceOrderService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/pending")
@Slf4j
@Tag(name = "待处理商品管理模块")
@Transactional(rollbackFor = Exception.class)
public class PendingServiceGoodController {


    @Resource
    private PendingServiceGoodService pendingServiceGoodService;


    @Resource
    private PendingServiceOrderService pendingServiceOrderService;



    /**
     * Web端管理员查询预约情况
     * @param bookingOrderQueryRequest  服务类商品预约情况查询请求体
     * @return 服务类商品预约情况列表
     */
    @PostMapping("/list/advance/detail")
    @Operation(summary = "Web端管理员查询预约情况", description = "参数：订单查询请求体，权限：管理员(admin, boss)，方法名：listOrder")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<List<PendingServiceGoodVO>> listBookingOrderDetail(@RequestBody BookingOrderQueryRequest bookingOrderQueryRequest) {
        if (bookingOrderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        QueryWrapper<PendingServiceGood> queryWrapper = pendingServiceGoodService.getQueryWrapper(bookingOrderQueryRequest);
        List<PendingServiceGood> pendingServiceGoodList = pendingServiceGoodService.list(queryWrapper);
        List<Long> pendingGoodIds = pendingServiceGoodList.stream().map(PendingServiceGood::getId).toList();
        QueryWrapper<PendingServiceOrder> orderQueryWrapper = new QueryWrapper<>();
        orderQueryWrapper.in("pendingId", pendingGoodIds);
        List<PendingServiceOrder> pendingServiceOrderList = pendingServiceOrderService.list(orderQueryWrapper);

        List<PendingServiceOrderVO> pendingServiceOrderVOS = pendingServiceOrderList.stream().map(pendingServiceOrder -> {
            PendingServiceOrderVO pendingServiceOrderVO = new PendingServiceOrderVO();
            BeanUtils.copyProperties(pendingServiceOrder, pendingServiceOrderVO);
            return pendingServiceOrderVO;
        }).toList();

        // 封装map集合（键：服务类商品待处理id，值：待处理服务类订单列表）
        Map<Long, List<PendingServiceOrderVO>> map = new HashMap<>();
        for (PendingServiceOrderVO pendingServiceOrderVO : pendingServiceOrderVOS) {
            Long pendingId = pendingServiceOrderVO.getPendingId();
            List<PendingServiceOrderVO> pendingServiceOrderVOList = map.get(pendingId);
            if (pendingServiceOrderVOList == null) {
                pendingServiceOrderVOList = new ArrayList<>();
            }
            pendingServiceOrderVOList.add(pendingServiceOrderVO);
            map.put(pendingId, pendingServiceOrderVOList);
        }

        List<PendingServiceGoodVO> pendingServiceGoodVOList = pendingServiceGoodList.stream().map(pendingServiceGood -> {
            PendingServiceGoodVO pendingServiceGoodVO = new PendingServiceGoodVO();
            BeanUtils.copyProperties(pendingServiceGood, pendingServiceGoodVO);
            Long goodId = pendingServiceGood.getId();
            List<PendingServiceOrderVO> pendingServiceOrderVOList = map.get(goodId);
            pendingServiceGoodVO.setPendingServiceOrderVOList(pendingServiceOrderVOList);
            return pendingServiceGoodVO;
        }).toList();

        return ResultUtils.success(pendingServiceGoodVOList);
    }


}
