package com.cultural.heritage.service.common;

import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;
import java.util.Map;
import java.util.function.Function;

public interface CommonService {


    /**
     * 取出一个集合的某个id列表, 去另外一个集合中筛选出属于这个id列表的集合
     */
    <T, R> List<R> findByOtherIds(List<T> sourceList, IService<R> genericService, Function<T, Long> getId);



    /**
     * 根据 ID 列表获取对应的实体列表
     */
    <T> List<T> findByMyIds(List<Long> ids, IService<T> genericService);



    /**
     * 从第一个集合中提取某个属性值，并用这些值作为查询条件去查询第二个集合的数据。
     * @param sourceList 原始集合（源数据）
     * @param service 要执行查询的 Service
     * @param sourceField 在原始集合中提取的字段名
     * @param targetField 在目标集合中的查询字段名
     * @param <T> 目标查询实体类型
     * @return 查询结果集合
     */
    <T> List<T> findBySourceFieldValues(List<?> sourceList, IService<T> service, String sourceField, String targetField);



    /**
     * 根据指定字段名和值，使用给定的服务对象查询对应的实体类列表。
     * @param <T> 实体类类型
     * @param fieldName 查询字段的名称
     * @param fieldValue 查询字段的值
     * @param service 用于执行查询的服务层对象
     * @return 返回符合条件的实体类列表（`List<T>`）。如果没有符合条件的记录，返回空列表。
     */
    <T> List<T> findByFieldValue(String fieldName, Object fieldValue, IService<T> service);



    /**
     * 根据多个字段和对应的值进行查询。
     * 该方法可以动态构建查询条件并执行查询。
     *
     * @param fieldConditions 查询条件的字段和值，使用Map存储
     * @param service         执行查询操作的服务对象（通常是MyBatis-Plus的 IService）
     * @return               返回查询结果的列表
     */
    <T> List<T> findByMultipleFieldValues(Map<String, Object> fieldConditions, IService<T> service);



    /**
     * 将一个类型的 List 转换为另一个类型的 List。
     * @param sourceList 源列表，类型为 T
     * @param targetClass 目标类型的 Class 对象
     * @param <T> 源类型
     * @param <R> 目标类型
     * @return 转换后的目标类型列表
     */
    <T, R> List<R> convertList(List<T> sourceList, Class<R> targetClass);






}
